"""Custom tkinter messagebox."""
import tkinter as tk
from tkinter import ttk
from tkinter import font
from pathlib import Path
from PIL import ImageTk, Image

from psiutils.constants import PAD

import text

icons = {
    'info': 'icon-info.png',
    'error': 'icon-error.png',
}


class MessageBox():
    def __init__(
            self,
            title: str = '',
            message: str = '',
            parent: tk.Frame = None,
            icon: str = 'info',
            ) -> None:
        self.root = tk.Toplevel(parent.root)
        self.title = title
        self.parent = parent
        self.icon = icon

        # tk variables
        self.message_text = tk.StringVar(value=message)

        # self.show()

    def show(self) -> None:
        root = self.root
        root.transient(self.parent.root)
        root.title(self.title)

        root.bind('<Control-o>', self._dismiss)

        path = Path(Path(__file__).parent, 'images', icons[self.icon])
        self.icon_image = ImageTk.PhotoImage(Image.open(path))

        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1)

        main_frame = self._main_frame(root)
        main_frame.grid(row=0, column=0, sticky=tk.NSEW, padx=PAD, pady=PAD)

        sizegrip = ttk.Sizegrip(root)
        sizegrip.grid(sticky=tk.SE)

    def _main_frame(self, master: tk.Frame) -> ttk.Frame:
        frame = ttk.Frame(master)
        frame.columnconfigure(1, weight=1)

        label = tk.Label(frame, image=self.icon_image)
        label.grid(row=0, column=0)

        label = ttk.Label(
            frame,
            textvariable=self.message_text,
            font=(font.nametofont('TkDefaultFont'), 12, 'bold')
            )
        label.grid(row=0, column=1, sticky=tk.NSEW, padx=PAD, pady=PAD)

        button_frame = self._button_frame(frame)
        button_frame.grid(row=1, column=0, columnspan=2,
                          sticky=tk.EW, padx=PAD, pady=PAD)

        return frame

    def _button_frame(self, master: tk.Frame) -> tk.Frame:
        frame = ttk.Frame(master)
        frame.rowconfigure(0, weight=1)
        frame.columnconfigure(0, weight=1)

        button = ttk.Button(
                frame,
                text=text.OK,
                command=self._dismiss,
                underline=0,
                )
        button.grid(row=0, column=0)
        return frame

    def _process(self, *args) -> None:
        ...

    def _dismiss(self, *args) -> None:
        self.root.destroy()


def showinfo(
        title: str = '',
        message: str = '',
        parent: tk.Frame = None,
        ) -> None:

    messagebox = MessageBox(
        title=title,
        message=message,
        parent=parent,
        icon='info',
        )

    messagebox.show()
    parent.root.wait_window(messagebox.root)


def showerror(
        title: str = '',
        message: str = '',
        parent: tk.Frame = None,
        ) -> None:

    messagebox = MessageBox(
        title=title,
        message=message,
        parent=parent,
        icon='error',
        )

    messagebox.show()
    parent.root.wait_window(messagebox.root)
